@@ -138,6 +138,10 @@ int mnt_lock_use_simplelock(struct libmnt_lock *ml, int enable)
 	ml->simplelock = enable ? 1 : 0;
 
 	sz = strlen(ml->lockfile);
+	assert(sz);
+
+	if (sz < 1)
+		return -EINVAL;
 
 	/* Change lock name:
 	 *
@@ -298,7 +302,7 @@ static int mnt_wait_mtab_lock(struct libmnt_lock *ml, struct flock *fl, time_t m
  *
  * Where does the link point to? Obvious choices are mtab and mtab~~.
  * HJLu points out that the latter leads to races. Right now we use
- * mtab~.<pid> instead.
+ * mtab~.pid instead.
  *
  *
  * The original mount locking code has used sleep(1) between attempts and
@@ -319,7 +323,7 @@ static int mnt_wait_mtab_lock(struct libmnt_lock *ml, struct flock *fl, time_t m
  * backwardly compatible code.
  *
  * Don't forget that this code has to be compatible with 3rd party mounts
- * (/sbin/mount.<foo>) and has to work with NFS.
+ * (/sbin/mount.foo) and has to work with NFS.
  * -- kzak@redhat.com [May-2009]
  */
 
@@ -585,7 +589,7 @@ void clean_lock(void)
 	mnt_free_lock(lock);
 }
 
-void sig_handler(int sig)
+void __attribute__((__noreturn__)) sig_handler(int sig)
 {
 	errx(EXIT_FAILURE, "\n%d: catch signal: %s\n", getpid(), strsignal(sig));
 }
